import Graphics.GD
import Data.Complex

----------------------------------------------------
------- This is to be modified by the user ---------
----------------------------------------------------


-- The size of the picture:
picSize :: Size
picSize = (500, 500)

-- The center of the fractal
xc :: Double
xc = -1.57

yc :: Double
yc = -0.035

-- The zoom ratio
zoom :: Double
zoom = 10


----------------------------------------------------
------- This is the rest of the program which ------
------- should not be modified by the user ---------
------- unless they want to play with the ----------
------- colors of the fractal ----------------------
----------------------------------------------------


-- Creating two new additional types
type Coordinate = (Double, Double) -- 2D coordinte
type Plane = ((Coordinate, Coordinate)) -- first coordinate represents the bottom left corner
                                        -- of the plane, the second coordinate represents the
                                        -- top right corner of the plane

-- The maximum number of iterations for the fractal function
maxIterations :: Int
maxIterations = 100

-- the width of the plane corresponding to the part of the fractal which will be drawn
xl :: Double
xl = 3 / zoom -- see Explanation

-- the height of the plane corresponding to the part of the fractal which will be drawn
yl :: Double
yl = 4 / zoom -- see Explanation1

-- Explanation1: the Overall image has a size of 3 x 4 on the complex plane.
--               If we zoom in by a ratio of "zoom", then the new size of the
--               complex plane that we are drawing is 3 / zoom and 4 / zoom.

-- left coordinate on the x axis:
x1 :: Double
x1 = xc - xl / 2 -- See Explanation2

-- right coordinate on the x axis:
x2 :: Double
x2 = x1 + xl -- see Explanation2

-- bottom coordinate on the y axis:
y1 :: Double
y1 = yc - yl / 2 -- see Explanation2

-- top coordinate on the y axis:
y2 :: Double
y2 = y1 + yl -- see Explanation2

-- Explanation2: the coordinates have to respect the following formulae:
--               x2 - x1 = width of the plane = xl
--               y2 - y1 = height of the plane = yl
--               (x1 + x2) / 2 = x coordinate for the center = xc
--               (y1 + y2) / 2 = y coordinate for the center = yc

-- creating the plane corresponding to the part of the fractal which will be drawn
plane :: Plane
plane = (bottomLeftCorner, topRightCorner)
      where bottomLeftCorner = (x1, y1)
            topRightCorner   = (x2, y2)

-- the function for getting the next value of z. z_n+1 = (|Re(z_n)| + i|Im(z_n)|)^2 + c, z0 = 0
shipFunction :: Int -> Complex Double -> Complex Double -> Int
shipFunction currentIteration z c = result
                                 where result = if currentIteration > maxIterations then 0
                                                else if magnitude next_z > 10 then currentIteration -- the bigger the magnitude, the bigger the fire
                                                else shipFunction (currentIteration + 1) next_z c
                                       next_z = (abs(realPart z) :+ abs(imagPart z))^2 + c

-- getting the number of iterations for a point
colorCoordinate :: Coordinate -> Color
colorCoordinate (x, y) = getColor nrIterations
                       where nrIterations  = shipFunction 0 startingValue (x :+ y)
                             startingValue = 0 :+ 0

-- getting a color for a point depending on the number of iterations
getColor :: Int -> Color
getColor nrIterations
    | nrIterations <= maxIterations = getColorForBounded nrIterations
    | otherwise                     = colorForEscaped

-- associate a color for a specific number of iterations for a bounded point
getColorForBounded :: Int -> Color
getColorForBounded x
    | x == 0 = rgb 0 0 0
 	| x <= 5 = rgb 20 20 20
 	| x <= 8 = rgb 30 30 30
 	| x <= 11 = rgb 50 50 50
 	| x <= 14 = rgb 70 70 70
    | x <= 17 = rgb 90 90 90
    | x <= 20 = rgb 110 110 110
    | x >= 20 && x <= 100 = rgb 195 195 195
    | otherwise          = rgb 112 128 144

-- associate a color for the escaped points (by default, it is black)
colorForEscaped :: Color
colorForEscaped = rgb 0 0 0

-- the main function which asks the user about the name of the new pictures
-- and after that it calls the function drawBurningShip which draws the fractal
main = do
    putStrLn "Enter the name of the new picture: "
    picName <- getLine
    drawBurningShip (picName ++ ".png")

-- this functions draws the fractal
drawBurningShip :: String -> IO ()
drawBurningShip name = do
                        result <- newImage picSize
                        mapM_ (\pixel -> setPixel pixel (colorCoordinate (getCoordinatesFrom plane pixel picSize)) result) $ pixels
                        savePngFile name result
                     where pixels = getPixels picSize

-- this function returns a list of all pixels from the image
getPixels :: Size -> [Point]
getPixels (width, height) = [(i, j) | i <- [0..width - 1], j <- [0..height - 1]]

-- this function associates a pixel with a coordinate in the plane
-- corresponding to the part of the fractal which will be drawn
getCoordinatesFrom :: Plane -> Point -> Size -> Coordinate
getCoordinatesFrom ((x1, y1), (x2, y2)) (x, y) (w, h) = (xr, yr)
                                                      where xr          = (planeWidth * (fromIntegral x / fromIntegral w)) + x1
                                                            yr          = (planeHeight * (fromIntegral y / fromIntegral h)) + y1
                                                            planeWidth  = x2 - x1
                                                            planeHeight = y2 - y1
